/*

ͻ
ZIDIP.C:   Input Impedance of a cylindrical dipole D with symmetrical   
           feeding, wire conductor radius R, and length L.              
                                                                        
             By Cristina & Fernando de Castro - November  2001          
ͼ

		ͻ
		                                     
		   This program should be compiled   
		         in DJGPP compiler           
		                                     
		ͼ
*/


/***************************************************************************
*
* HEADERS:
*
****************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<process.h>
#include<math.h>
#include<float.h>
#include<conio.h>
#include<ctype.h>
#include<string.h>
#include<time.h>
#include<dos.h>
#include<fcntl.h>
#include<share.h>

/*****************************************************************************
*
* PROGRAM DEFINITIONS:
*
*****************************************************************************/
#define BUFSIZE 0xFFFF /* file I/O buffer size: 0xFFFF = 64Kb */
#define SMALLBUFSIZE  0x03FF /* file I/O small buffer size 0x03FF = 1Kb  */

/*****************************************************************************
*
* COMPILING DIRECTIVES:
*
*****************************************************************************/

//#define DEBUG_FILE "dbg.txt" /* if defined open ascii debug file */

/*****************************************************************************
*
* CONSTANTS:
*
*****************************************************************************/

#define EPS 6.0e-8
#define EULER 0.5772156649 /* Euler constant */
#define SICIMAXIT 1000 /* Max iterations allowed in SiCi() */
#define FPMIN 1.0e-30
#define TMIN 2.0
#define TRUE 1

#define MAXFLOAT 3.4E38 /* 32 bits DJGPP maximum float */
#define MINABSFLOAT 1E-45 /* 32 bits DJGPP minimum float absolute value regarded as non-zero */

#define MAXEXPARG 88.7 /* 88.7 = ln( 3.4x10^38): max argument value to avoid
			float overflow in function exp() */

#define BEL 7  /* ascii BEL */


/***************************************************************************
*
* DATA TYPE STRUCTURES:
*
****************************************************************************/

typedef struct ComplexNumber {double r,i;} complex;


typedef enum {

  NO_EVENT,           /*  " ", */
  MEMORY_ERR,         /*  "Not enough memory", */
  READ_OPEN_ERR,      /*  "I can't find the file", */
  READ_ERR,           /*  "Error reading file", */
  WRITE_OPEN_ERR,     /*  "I can't open the file", */
  WRITE_ERR,          /*  "I can't write the file", */
  BUFFER_ERR,         /*  "I can't attach buffer to file", */
  SIZE_ERR,           /*  "Specified size doesn't match data size in file", */
  MAXIT_ERR,          /*  "Exceeded max number of iterations in function", */
  LOGIC_ERR,          /*  "Logic error in function", */
  READ_NNET_ERR,      /*  "Incompatible neural network in file", */
  MTXDIM_ERR,         /*  "Invalid matrix dimensions in file", */
  PARF_ERR,           /*  "Wrong parameter file", */
  GFTD_ERR,           /*  "I can't get time & date info from file", */
  MAXFLOAT_READ_ERR,  /*  "Exceeded float maximum absolute value reading file" */
  /* ---*/
}
EVENT;



/*****************************************************************************
*
* MACROS:
*
*****************************************************************************/
static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

#define SIGNUM(a) ((a) >= 0.0 ? 1.0 : -1.0)

#define ONE Complex(1.0,0.0)

/***************************************************************************
*
* FUNCTION PROTOTYPES:
*
****************************************************************************/
inline void SiCi(double *Si, double *Ci,double x);
inline complex Cadd(complex a, complex b);
inline complex Cmul(complex a, complex b);
inline complex Complex(double re, double im);
inline complex Cdiv(complex a, complex b);
inline complex RCmul(double x, complex a);
inline double Pythag(double a, double b);
complex ZIDIP(double L, double RAI, double LA);
void Quit(EVENT Evt, char *Message);

/***************************************************************************
*
* GLOBAL VARIABLES:
*
****************************************************************************/


/***************************************************************************
*
* MAIN():
*
****************************************************************************/
int main(int argc, char *argv[])
{

double Length,WireRadius,Lambda;
complex Zin;

/*         0        1              2         3       */
if(argc!=4){
puts("USE: Zi_CyDip WireRadius[mm] Lambda[m] Length[m]\n");
puts("     Zi_CyDip V1.7 by Cristina & Fernando DeCastro - decastro@ee.pucrs.br\n");
puts("     Input impedance  of a cylindrical  dipole with  symmetrical  feeding, full");
puts("     length  Length,  cylindrical  wire  radius  WireRadius  and  operating  at");
puts("     wavelength Lambda. Based upon The Biconical Antenna Perturbation Method by");
puts("     Schelkunoff.");
exit(0);


exit(0);
}


WireRadius=(1E-3)*atof(argv[1]);
Lambda=atof(argv[2]);
Length=atof(argv[3]);

Zin=ZIDIP(0.5*Length,WireRadius,Lambda);

#if(1)
printf("\nCylindrical wire radius: %8.6g [mm]\n",(1E3)*WireRadius);
printf("Dipole full length: %8.6g [m]\n",Length);
printf("Operating wavelength: %8.6g [m]\n",Lambda);
#endif

printf("\nZin=%8.6g%+8.6gi [ohm] (referred to the input terminals)\n\n",Zin.r,Zin.i);

return 0;
}

/*****************************************************************************
* FUNC: void Quit(EVENT Evt, char *Message)
*
* DESC: Print Event message and exit to shell.
******************************************************************************/
void Quit(EVENT Evt, char *Message)
{

static char *Event[] = {

/*  NO_EVENT */       " ",
/*  MEMORY_ERR */     "Not enough memory",
/*  READ_OPEN_ERR */  "I can't find the file",
/*  READ_ERR */       "Error reading file",
/*  WRITE_OPEN_ERR */ "I can't open the file",
/*  WRITE_ERR */      "I can't write the file",
/*  BUFFER_ERR */     "I can't attach buffer to file",
/*  SIZE_ERR */       "Specified size doesn't match data size in file",
/*  MAXIT_ERR */      "Exceeded max number of iterations in function",
/*  LOGIC_ERR */      "Logic error in function",
/*  READ_NNET_ERR */  "Incompatible neural network in file",
/*  MTXDIM_ERR */     "Invalid matrix dimensions in file",
/*  PARF_ERR  */      "Wrong parameter file",
/*  GFTD_ERR  */      "I can't get time & date info from file",
/*  MAXFLOAT_READ_ERR */"Exceeded float maximum absolute value reading file",
/*---------*/

};


	system("cls");

	if (Evt != NO_EVENT) {


		if(Message[0]){
		fprintf (stderr,"%s %s!\n", Event[Evt], Message);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}
		else{
		fprintf (stderr,"%s!\n", Event[Evt]);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}

	}
	exit (0);
}


/*****************************************************************************
* FUNC: inline void SiCi(double *Si, double *Ci,double x)
*
* DESC: The Sine Integral of x and the Cosine Inetgral of x are respectively
*       returned in *Si and *Ci.
******************************************************************************/
inline void SiCi(double *Si, double *Ci,double x)
{
	int i,k,odd;
	double a,err,fact,sign,sum,sumc,sums,t,term;
	complex h,b,c,d,del;
	t=fabs(x);
	if (t == 0.0) {
		*Si=0.0;
		*Ci = -1.0/FPMIN;
		return;
	}
	if (t > TMIN) {
		b=Complex(1.0,t);
		c=Complex(1.0/FPMIN,0.0);
		d=h=Cdiv(ONE,b);
		for (i=2;i<=SICIMAXIT;i++) {
			a = -(i-1)*(i-1);
			b=Cadd(b,Complex(2.0,0.0));
			d=Cdiv(ONE,Cadd(RCmul(a,d),b));
			c=Cadd(b,Cdiv(Complex(a,0.0),c));
			del=Cmul(c,d);
			h=Cmul(h,del);
			if (fabs(del.r-1.0)+fabs(del.i) < EPS) break;
		}
		if (i > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		h=Cmul(Complex(cos(t),-sin(t)),h);
		*Ci = -h.r;
		*Si=M_PI_2+h.i;
	} else {
		if (t < sqrt(FPMIN)) {
			sumc=0.0;
			sums=t;
		} else {
			sum=sums=sumc=0.0;
			sign=fact=1.0;
			odd=TRUE;
			for (k=1;k<=SICIMAXIT;k++) {
				fact *= t/k;
				term=fact/k;
				sum += sign*term;
				err=term/fabs(sum);
				if (odd) {
					sign = -sign;
					sums=sum;
					sum=sumc;
				} else {
					sumc=sum;
					sum=sums;
				}
				if (err < EPS) break;
				odd=!odd;
			}
			if (k > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		}
		*Si=sums;
		*Ci=sumc+log(t)+EULER;
	}
	if (x < 0.0) *Si = -(*Si);
}


/*****************************************************************************
* FUNC: inline complex Cadd(complex a, complex b)
*
* DESC: Return the sum of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cadd(complex a, complex b)
{
	complex c;
	c.r=a.r+b.r;
	c.i=a.i+b.i;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Cmul(complex a, complex b)
*
* DESC: Return the product of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cmul(complex a, complex b)
{
	complex c;
	c.r=a.r*b.r-a.i*b.i;
	c.i=a.i*b.r+a.r*b.i;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Complex(complex a, complex b)
*
* DESC: Return complex number re+j*im.
******************************************************************************/
inline complex Complex(double re, double im)
{
	complex c;
	c.r=re;
	c.i=im;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Cdiv(complex a, complex b)
*
* DESC: Return the quotient of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cdiv(complex a, complex b)
{
	complex c;
	double r,den;
	if (fabs(b.r) >= fabs(b.i)) {
		r=b.i/b.r;
		den=b.r+r*b.i;
		c.r=(a.r+r*a.i)/den;
		c.i=(a.i-r*a.r)/den;
	} else {
		r=b.r/b.i;
		den=b.i+r*b.r;
		c.r=(a.r*r+a.i)/den;
		c.i=(a.i*r-a.r)/den;
	}
	return c;
}


/*****************************************************************************
* FUNC: inline complex RCmul(double x, complex a)
*
* DESC: Return the product of real 'x' by complex 'a'.
******************************************************************************/
inline complex RCmul(double x, complex a)
{
	complex c;
	c.r=x*a.r;
	c.i=x*a.i;
	return c;
}


/*****************************************************************************
* FUNC: inline double Pythag(double a, double b)
*
* DESC: Compute (a^2+b^2)^{1/2} without overflow
*****************************************************************************/
inline double Pythag(double a, double b)
{
	double absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}

/*****************************************************************************
* FUNC: complex ZIDIP(double L, double RA, double LA)
*
* DESC: Input Impedance of a cylindrical dipole with symmetrical feeding
*
* INPUT PARAMETERS:   L    - Half-length of Dipole 1 in meters.
*                     RAI  - Dipole cylindrical wire radius in meters.
*                     LA   - Operation wavelength (lambda) in meters.
*
* NOTE 1: Based upon The Biconical Antenna Perturbation Method (by Schelkunoff)
*
*                    L        L
*              __________   __________
*      RAI -< [__________] [__________]
*                        | |
*                        o o
*
* NOTE 2: The method is precise only when RAI is less than approximately
*         (1/100) of the operating wavelength LA.
*
******************************************************************************/
complex ZIDIP(double L, double RAI, double LA)
{

complex ZIN,ZID,ZI;
double LSA,L2LA,L2BL,LBL,M,N;
double B,BL,DBL,QBL;
double SDBL,CDBL,SQBL,CQBL;
double R1,R2,R3,RA;
double X1,X2,X3,XA;
double ZOM;
double SBL,CBL;
double S2BL,C2BL;

B=2*M_PI/LA;

BL=B*L;

DBL=2*BL;

QBL=4*BL;

LSA=L/RAI;

L2LA=log(2*LSA);

L2BL=log(DBL);

LBL=log(BL);

SBL=sin(BL);

CBL=cos(BL);

S2BL=sin(DBL);

C2BL=cos(DBL);

SiCi(&SDBL,&CDBL,DBL);

SiCi(&SQBL,&CQBL,QBL);

R1=EULER+L2BL-CDBL;

R2=0.5*S2BL*(SQBL-2*SDBL);

R3=0.5*C2BL*(EULER+LBL+CQBL-2*CDBL);

RA=60*(R1+R2+R3);

X1=SDBL;

X2=0.5*S2BL*(EULER+LBL-CQBL);

X3=0.5*C2BL*SQBL;

XA=60*(X1-X2-X3);

M=60*(L2BL-CDBL+EULER-1+C2BL);

N=60*(SDBL-S2BL);

ZOM=120*(L2LA-1);

ZIN=Complex(RA*SBL,(XA-N)*SBL-(ZOM-M)*CBL);

ZID=Complex((ZOM+M)*SBL+(XA+N)*CBL,-(RA*CBL));

ZI=Cdiv(RCmul(ZOM,ZIN),ZID);

return ZI;

}

