/*

ͻ
Zm_CyPDS.C: Mutual Impedance of 2 parallel dipoles D1 and D2 with       
            symmetrical feeding, lengths Length1 and Length2 and        
            cylindrical wire conductors radii Radius1 and Radius2.      
                                                                        
             By Cristina & Fernando de Castro - November  2001          
ͼ

		ͻ
		                                     
		   This program should be compiled   
		         in DJGPP compiler           
		                                     
		ͼ
*/


/***************************************************************************
*
* HEADERS:
*
****************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<process.h>
#include<math.h>
#include<float.h>
#include<conio.h>
#include<ctype.h>
#include<string.h>
#include<time.h>
#include<dos.h>
#include<fcntl.h>
#include<share.h>

/*****************************************************************************
*
* PROGRAM DEFINITIONS:
*
*****************************************************************************/
#define BUFSIZE 0xFFFF /* file I/O buffer size: 0xFFFF = 64Kb */
#define SMALLBUFSIZE  0x03FF /* file I/O small buffer size 0x03FF = 1Kb  */

/*****************************************************************************
*
* COMPILING DIRECTIVES:
*
*****************************************************************************/

//#define DEBUG_FILE "dbg.txt" /* if defined open ascii debug file */

/*****************************************************************************
*
* CONSTANTS:
*
*****************************************************************************/

#define EPS 6.0e-8
#define EULER 0.5772156649 /* Euler constant */
#define SICIMAXIT 1000 /* Max iterations allowed in SiCi() */
#define FPMIN 1.0e-30
#define TMIN 2.0
#define TRUE 1

#define MAXFLOAT 3.4E38 /* 32 bits DJGPP maximum float */
#define MINABSFLOAT 1E-45 /* 32 bits DJGPP minimum float absolute value regarded as non-zero */

#define MAXEXPARG 88.7 /* 88.7 = ln( 3.4x10^38): max argument value to avoid
			float overflow in function exp() */

#define BEL 7  /* ascii BEL */


/***************************************************************************
*
* DATA TYPE STRUCTURES:
*
****************************************************************************/
typedef struct ComplexNumber {double r,i;} complex;


typedef enum {

  NO_EVENT,           /*  " ", */
  MEMORY_ERR,         /*  "Not enough memory", */
  READ_OPEN_ERR,      /*  "I can't find the file", */
  READ_ERR,           /*  "Error reading file", */
  WRITE_OPEN_ERR,     /*  "I can't open the file", */
  WRITE_ERR,          /*  "I can't write the file", */
  BUFFER_ERR,         /*  "I can't attach buffer to file", */
  SIZE_ERR,           /*  "Specified size doesn't match data size in file", */
  MAXIT_ERR,          /*  "Exceeded max number of iterations in function", */
  LOGIC_ERR,          /*  "Logic error in function", */
  READ_NNET_ERR,      /*  "Incompatible neural network in file", */
  MTXDIM_ERR,         /*  "Invalid matrix dimensions in file", */
  PARF_ERR,           /*  "Wrong parameter file", */
  GFTD_ERR,           /*  "I can't get time & date info from file", */
  MAXFLOAT_READ_ERR,  /*  "Exceeded float maximum absolute value reading file" */
  /* ---*/
}
EVENT;



/*****************************************************************************
*
* MACROS:
*
*****************************************************************************/
static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

#define SIGNUM(a) ((a) >= 0.0 ? 1.0 : -1.0)

#define ONE Complex(1.0,0.0)

/***************************************************************************
*
* FUNCTION PROTOTYPES:
*
****************************************************************************/
inline void SiCi(double *Si, double *Ci,double x);
inline complex Cadd(complex a, complex b);
inline complex Cmul(complex a, complex b);
inline complex Complex(double re, double im);
inline complex Cdiv(complex a, complex b);
inline complex RCmul(double x, complex a);
inline double Pythag(double a, double b);
complex ZMPDS(double L1, double L2, double RAI1, double RAI2, double D,double LA);
void Quit(EVENT Evt, char *Message);

/***************************************************************************
*
* GLOBAL VARIABLES:
*
****************************************************************************/


/***************************************************************************
*
* MAIN():
*
****************************************************************************/
int main(int argc, char *argv[])
{

double Length1,Length2,Radius1,Radius2,Spacing,Lambda;
complex Z12;


if(argc!=7){
/*         0        1          2          3           4           5       6      */
puts("USE: Zm_CyPDS Length1[m] Length2[m] Radius1[mm] Radius2[mm] Spac[m] Lambda[m]\n");
puts("     Zm_PDS V1.7 by Cristina & Fernando DeCastro - decastro@ee.pucrs.br\n");
puts("     Mutual impedance of 2 parallel dipoles D1 and D2 with symmetrical feeding,");
puts("     full lengths  Length1 and Length2,  cylindrical wires  radii  Radius1  and");
puts("     Radius2,  spatially separated by Spac  and operating at wavelength Lambda.");
puts("     Based upon The Biconical Antenna Perturbation Method by Schelkunoff.");
exit(0);
}


Length1=atof(argv[1]);
Length2=atof(argv[2]);
Radius1=(1E-3)*atof(argv[3]);
Radius2=(1E-3)*atof(argv[4]);
Spacing=atof(argv[5]);
Lambda=atof(argv[6]);

Z12=ZMPDS(0.5*Length1,0.5*Length2,Radius1,Radius2,Spacing,Lambda);

#if(1)
printf("\nFull length of dipole D1: %8.6g [m]\n",Length1);
printf("Full length of dipole D2: %8.6g [m]\n",Length2);
printf("D1 cylindrical wire radius: %8.6g [mm]\n",(1E3)*Radius1);
printf("D2 cylindrical wire radius: %8.6g [mm]\n",(1E3)*Radius2);
printf("Spacing between parallel dipoles D1 and D2: %8.6g [m]\n",Spacing);
printf("Operating wavelength: %8.6g [m]\n",Lambda);
#endif

printf("\nZ12=%8.6g%+8.6gi [ohm] (referred to D1 input terminals)\n\n",Z12.r,Z12.i);

return 0;
}

/*****************************************************************************
* FUNC: void Quit(EVENT Evt, char *Message)
*
* DESC: Print Event message and exit to shell.
******************************************************************************/
void Quit(EVENT Evt, char *Message)
{

static char *Event[] = {

/*  NO_EVENT */       " ",
/*  MEMORY_ERR */     "Not enough memory",
/*  READ_OPEN_ERR */  "I can't find the file",
/*  READ_ERR */       "Error reading file",
/*  WRITE_OPEN_ERR */ "I can't open the file",
/*  WRITE_ERR */      "I can't write the file",
/*  BUFFER_ERR */     "I can't attach buffer to file",
/*  SIZE_ERR */       "Specified size doesn't match data size in file",
/*  MAXIT_ERR */      "Exceeded max number of iterations in function",
/*  LOGIC_ERR */      "Logic error in function",
/*  READ_NNET_ERR */  "Incompatible neural network in file",
/*  MTXDIM_ERR */     "Invalid matrix dimensions in file",
/*  PARF_ERR  */      "Wrong parameter file",
/*  GFTD_ERR  */      "I can't get time & date info from file",
/*  MAXFLOAT_READ_ERR */"Exceeded float maximum absolute value reading file",
/*---------*/

};


	system("cls");

	if (Evt != NO_EVENT) {


		if(Message[0]){
		fprintf (stderr,"%s %s!\n", Event[Evt], Message);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}
		else{
		fprintf (stderr,"%s!\n", Event[Evt]);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}

	}
	exit (0);
}


/*****************************************************************************
* FUNC: inline void SiCi(double *Si, double *Ci,double x)
*
* DESC: The Sine Integral of x and the Cosine Inetgral of x are respectively
*       returned in *Si and *Ci.
******************************************************************************/
inline void SiCi(double *Si, double *Ci,double x)
{
	int i,k,odd;
	double a,err,fact,sign,sum,sumc,sums,t,term;
	complex h,b,c,d,del;
	t=fabs(x);
	if (t == 0.0) {
		*Si=0.0;
		*Ci = -1.0/FPMIN;
		return;
	}
	if (t > TMIN) {
		b=Complex(1.0,t);
		c=Complex(1.0/FPMIN,0.0);
		d=h=Cdiv(ONE,b);
		for (i=2;i<=SICIMAXIT;i++) {
			a = -(i-1)*(i-1);
			b=Cadd(b,Complex(2.0,0.0));
			d=Cdiv(ONE,Cadd(RCmul(a,d),b));
			c=Cadd(b,Cdiv(Complex(a,0.0),c));
			del=Cmul(c,d);
			h=Cmul(h,del);
			if (fabs(del.r-1.0)+fabs(del.i) < EPS) break;
		}
		if (i > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		h=Cmul(Complex(cos(t),-sin(t)),h);
		*Ci = -h.r;
		*Si=M_PI_2+h.i;
	} else {
		if (t < sqrt(FPMIN)) {
			sumc=0.0;
			sums=t;
		} else {
			sum=sums=sumc=0.0;
			sign=fact=1.0;
			odd=TRUE;
			for (k=1;k<=SICIMAXIT;k++) {
				fact *= t/k;
				term=fact/k;
				sum += sign*term;
				err=term/fabs(sum);
				if (odd) {
					sign = -sign;
					sums=sum;
					sum=sumc;
				} else {
					sumc=sum;
					sum=sums;
				}
				if (err < EPS) break;
				odd=!odd;
			}
			if (k > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		}
		*Si=sums;
		*Ci=sumc+log(t)+EULER;
	}
	if (x < 0.0) *Si = -(*Si);
}


/*****************************************************************************
* FUNC: inline complex Cadd(complex a, complex b)
*
* DESC: Return the sum of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cadd(complex a, complex b)
{
	complex c;
	c.r=a.r+b.r;
	c.i=a.i+b.i;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Cmul(complex a, complex b)
*
* DESC: Return the product of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cmul(complex a, complex b)
{
	complex c;
	c.r=a.r*b.r-a.i*b.i;
	c.i=a.i*b.r+a.r*b.i;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Complex(complex a, complex b)
*
* DESC: Return complex number re+j*im.
******************************************************************************/
inline complex Complex(double re, double im)
{
	complex c;
	c.r=re;
	c.i=im;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Cdiv(complex a, complex b)
*
* DESC: Return the quotient of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cdiv(complex a, complex b)
{
	complex c;
	double r,den;
	if (fabs(b.r) >= fabs(b.i)) {
		r=b.i/b.r;
		den=b.r+r*b.i;
		c.r=(a.r+r*a.i)/den;
		c.i=(a.i-r*a.r)/den;
	} else {
		r=b.r/b.i;
		den=b.i+r*b.r;
		c.r=(a.r*r+a.i)/den;
		c.i=(a.i*r-a.r)/den;
	}
	return c;
}


/*****************************************************************************
* FUNC: inline complex RCmul(double x, complex a)
*
* DESC: Return the product of real 'x' by complex 'a'.
******************************************************************************/
inline complex RCmul(double x, complex a)
{
	complex c;
	c.r=x*a.r;
	c.i=x*a.i;
	return c;
}


/*****************************************************************************
* FUNC: inline double Pythag(double a, double b)
*
* DESC: Compute (a^2+b^2)^{1/2} without overflow
*****************************************************************************/
inline double Pythag(double a, double b)
{
	double absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}


/*****************************************************************************
* FUNC: complex ZMPDS(double L1, double L2, double RAI1, double RAI2, double D,double LA)
*
* DESC: Mutual Impedance of 2 Parallel Dipoles with Symmetrical Feeding.
*
* INPUT PARAMETERS:   L1   - Half-length of Dipole 1 in meters.
*                     L2   - Half-length of Dipole 2 in meters.
*                     RAI1 - Dipole 1 conductor cylindrical wire radius in meters.
*                     RAI2 - Dipole 1 conductor cylindrical wire radius in meters.
*                     D    - Distance between Dipoles 1 and 2 in meters.
*                     LA   - Operation wavelength (lambda) in meters.
*
* NOTE 1: Based upon The Biconical Antenna Perturbation Method (by Schelkunoff)
*
*
*
*                   L1        L1
*              __________   __________
*     RAI1 -< [__________] [__________] ....................
*                        | |                               |
*                        o o                               |
*                                                          |
*                         |                                D
*                         |                                |
*                   L2    |    L2                          |
*                ________   ________                       |
*       RAI2 -< [________] [________] .....................|
*                        | |
*                        o o
*
*
* NOTE 2: The method is precise only when RAI1 and RAI2 are both
*         less than approximately (1/100) of the operating wavelength LA.
*
******************************************************************************/
complex ZMPDS(double L1, double L2, double RAI1, double RAI2, double D,double LA)
{

complex ZA1,ZA2,I1,I2,Z12A,Z12;
double LSA1,LSA2,L2LA1,L2LA2,L2BL1,L2BL2,LBL1,LBL2,M1,M2,N1,N2;
double B,BL1,BL2,DBL1,DBL2,QBL1,QBL2;
double SDBL1,CDBL1,SDBL2,CDBL2,SQBL1,CQBL1,SQBL2,CQBL2;
double R11,R12,R21,R22,R31,R32,RA1,RA2;
double X11,X12,X21,X22,X31,X32,XA1,XA2;
double ZOM1,ZOM2,R0,R1,R2,R0L,BL12,BL21;
double A1,A2,A3,A4,A5,A6,A7,A8,A9;
double S1,S2,S3,S4,S5,S6,S7,S8,S9;
double C1,C2,C3,C4,C5,C6,C7,C8,C9;
double O1,O2,O3,O4,CO12,CO21,SI12,SI21,R12A,Q1,Q2,Q3,Q4,X12A;
double SBL1,SBL2,CBL1,CBL2;
double S2BL1,S2BL2,C2BL1,C2BL2;

B=2*M_PI/LA;

BL1=B*L1;

BL2=B*L2;

DBL1=2*BL1;

DBL2=2*BL2;

QBL1=4*BL1;

QBL2=4*BL2;

LSA1=L1/RAI1;

LSA2=L2/RAI2;

L2LA1=log(2*LSA1);

L2LA2=log(2*LSA2);

L2BL1=log(DBL1);

L2BL2=log(DBL2);

LBL1=log(BL1);

LBL2=log(BL2);

SBL1=sin(BL1);

SBL2=sin(BL2);

CBL1=cos(BL1);

CBL2=cos(BL2);

S2BL1=sin(DBL1);

S2BL2=sin(DBL2);

C2BL1=cos(DBL1);

C2BL2=cos(DBL2);

SiCi(&SDBL1,&CDBL1,DBL1);

SiCi(&SDBL2,&CDBL2,DBL2);

SiCi(&SQBL1,&CQBL1,QBL1);

SiCi(&SQBL2,&CQBL2,QBL2);

R11=EULER+L2BL1-CDBL1;

R12=EULER+L2BL2-CDBL2;

R21=0.5*S2BL1*(SQBL1-2*SDBL1);

R22=0.5*S2BL2*(SQBL2-2*SDBL2);

R31=0.5*C2BL1*(EULER+LBL1+CQBL1-2*CDBL1);

R32=0.5*C2BL2*(EULER+LBL2+CQBL2-2*CDBL2);

RA1=60*(R11+R21+R31);

RA2=60*(R12+R22+R32);

X11=SDBL1;

X12=SDBL2;

X21=0.5*S2BL1*(EULER+LBL1-CQBL1);

X22=0.5*S2BL2*(EULER+LBL2-CQBL2);

X31=0.5*C2BL1*SQBL1;

X32=0.5*C2BL2*SQBL2;

XA1=60*(X11-X21-X31);

XA2=60*(X12-X22-X32);

ZA1=Complex(RA1,XA1);

ZA2=Complex(RA2,XA2);

M1=60*(L2BL1-CDBL1+EULER-1+C2BL1);

M2=60*(L2BL2-CDBL2+EULER-1+C2BL2);

N1=60*(SDBL1-S2BL1);

N2=60*(SDBL2-S2BL2);

ZOM1=120*(L2LA1-1);

ZOM2=120*(L2LA2-1);

I1=Complex((1+M1/ZOM1)*SBL1+((N1+ZA1.i)/ZOM1)*CBL1,((N1-ZA1.r)/ZOM1)*CBL1);

I2=Complex((1+M2/ZOM2)*SBL2+((N2+ZA2.i)/ZOM2)*CBL2,((N2-ZA2.r)/ZOM2)*CBL2);

R0=B*Pythag(D,L2);

R1=B*Pythag(D,L1+L2);

R2=B*Pythag(D,L2-L1);

R0L=B*Pythag(D,L1);

BL12=BL1+BL2;

BL21=BL2-BL1;

A1=B*D;

A2=R0+BL2;

A3=R0-BL2;

A4=R1-BL1-BL2;

A5=R1+BL1+BL2;

A6=R0L-BL1;

A7=R0L+BL1;

A8=R2-BL2+BL1;

A9=R2+BL2-BL1;

SiCi(&S1,&C1,A1);

SiCi(&S2,&C2,A2);

SiCi(&S3,&C3,A3);

SiCi(&S4,&C4,A4);

SiCi(&S5,&C5,A5);

SiCi(&S6,&C6,A6);

SiCi(&S7,&C7,A7);

SiCi(&S8,&C8,A8);

SiCi(&S9,&C9,A9);

O1=2*C1-C2-C3+C4+C5-C6-C7;

O2=2*C1-C2-C3+C8+C9-C6-C7;

O3=S3-S2+S5-S4+S6-S7;

O4=S3-S2+S9-S8+S7-S6;

CO12=cos(BL12);

CO21=cos(BL21);

SI12=sin(BL12);

SI21=sin(BL21);

R12A=30*(CO12*O1+CO21*O2+SI12*O3+SI21*O4);

Q1=S3+S2-2*S1-S4-S5+S7+S6;

Q2=S3+S2-2*S1-S8-S9+S7+S6;

Q3=C3-C2-C4+C5+C6-C7;

Q4=C3-C2-C8-C9+C7-C6;

X12A=30*(CO12*Q1+CO21*Q2+SI12*Q3+SI21*Q4);

Z12A=Complex(R12A,X12A);

Z12=Cdiv(Z12A,Cmul(I1,I2));

return Z12;

}

