/*

ͻ
Zm_EchDip.C: Mutual Impedance of echelon thin dipoles D1 and D2 with    
             symmetrical feeding and lengths Length1 and Length2        
                                                                        
             By Cristina and Fernando de Castro - February  2002        
ͼ

		ͻ
		                                     
		   This program should be compiled   
		         in DJGPP compiler           
		                                     
		ͼ
*/


/***************************************************************************
*
* HEADERS:
*
****************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<process.h>
#include<math.h>
#include<float.h>
#include<conio.h>
#include<ctype.h>
#include<string.h>
#include<time.h>
#include<dos.h>
#include<fcntl.h>
#include<share.h>

/*****************************************************************************
*
* PROGRAM DEFINITIONS:
*
*****************************************************************************/
#define BUFSIZE 0xFFFF /* file I/O buffer size: 0xFFFF = 64Kb */
#define SMALLBUFSIZE  0x03FF /* file I/O small buffer size 0x03FF = 1Kb  */

/*****************************************************************************
*
* COMPILING DIRECTIVES:
*
*****************************************************************************/

//#define DEBUG_FILE "dbg.txt" /* if defined open ascii debug file */

/*****************************************************************************
*
* CONSTANTS:
*
*****************************************************************************/
#define EPS 6.0e-8
#define EULER 0.5772156649 /* Euler constant */
#define SICIMAXIT 1000 /* Max iterations allowed in SiCi() */
#define FPMIN 1.0e-30
#define TMIN 2.0
#define TRUE 1

#define MAXFLOAT 3.4E38 /* 32 bits DJGPP maximum float */
#define MINABSFLOAT 1E-45 /* 32 bits DJGPP minimum float absolute value regarded as non-zero */

#define MAXEXPARG 88.7 /* 88.7 = ln( 3.4x10^38): max argument value to avoid
			float overflow in function exp() */

#define BEL 7  /* ascii BEL */

static double _2PI=2*M_PI; /* 2pi */

/***************************************************************************
*
* DATA TYPE STRUCTURES:
*
****************************************************************************/

typedef struct ComplexNumber {double r,i;} complex;


typedef enum {

  NO_EVENT,           /*  " ", */
  MEMORY_ERR,         /*  "Not enough memory", */
  READ_OPEN_ERR,      /*  "I can't find the file", */
  READ_ERR,           /*  "Error reading file", */
  WRITE_OPEN_ERR,     /*  "I can't open the file", */
  WRITE_ERR,          /*  "I can't write the file", */
  BUFFER_ERR,         /*  "I can't attach buffer to file", */
  SIZE_ERR,           /*  "Specified size doesn't match data size in file", */
  MAXIT_ERR,          /*  "Exceeded max number of iterations in function", */
  LOGIC_ERR,          /*  "Logic error in function", */
  READ_NNET_ERR,      /*  "Incompatible neural network in file", */
  MTXDIM_ERR,         /*  "Invalid matrix dimensions in file", */
  PARF_ERR,           /*  "Wrong parameter file", */
  GFTD_ERR,           /*  "I can't get time & date info from file", */
  MAXFLOAT_READ_ERR,  /*  "Exceeded float maximum absolute value reading file" */
  /* ---*/
}
EVENT;



/*****************************************************************************
*
* MACROS:
*
*****************************************************************************/
static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

#define SIGNUM(a) ((a) >= 0.0 ? 1.0 : -1.0)

#define ONE Complex(1.0,0.0)


/***************************************************************************
*
* FUNCTION PROTOTYPES:
*
****************************************************************************/
inline void SiCi(double *Si, double *Ci,double x);
inline complex Cadd(complex a, complex b);
inline complex Csub(complex a, complex b);
inline complex Cmul(complex a, complex b);
inline complex Complex(double re, double im);
inline complex Cdiv(complex a, complex b);
inline complex RCmul(double x, complex a);
inline double Pythag(double a, double b);
void Quit(EVENT Evt, char *Message);
complex ZM_EchelonDipoles(float SL1, float SL2, float GSX, float GSY);

/***************************************************************************
*
* GLOBAL VARIABLES:
*
****************************************************************************/


/***************************************************************************
*
* MAIN():
*
****************************************************************************/
int main(int argc, char *argv[])
{

float Length1,Length2,XSpacing,YSpacing,Lambda;
complex Z12;


if(argc!=6){
/*         0         1          2          3               4              5              */
puts("USE: Zm_EchDip Length1[m] Length2[m] ParallelSpac[m] PerpendSpac[m] Lambda[m]\n");
puts("     Zm_EchDip V1.7 by Cristina & Fernando DeCastro - decastro@ee.pucrs.br\n");
puts("     Mutual impedance of  2 echelon thin dipoles  D1 and D2   with  symmetrical");
puts("     feeding, full  lengths   Length1  and  Length2, spatially   separated   by");
puts("     ParallelSpac in the E-Plane  and PerpendSpac in the H-Plane  and operating");
puts("     at wavelength Lambda. Based upon The Induced FEM Method.");
exit(0);

exit(0);
}

Length1=atof(argv[1]);
Length2=atof(argv[2]);
XSpacing=atof(argv[3]);
YSpacing=atof(argv[4]);
Lambda=atof(argv[5]);

Z12=ZM_EchelonDipoles(0.5*Length1/Lambda,0.5*Length2/Lambda,XSpacing/Lambda,YSpacing/Lambda);


#if(1)
printf("\nLength of dipole D1: %8.6g [m]\n",Length1);
printf("Length of dipole D2: %8.6g [m]\n",Length2);
printf("Parallel (E-Plane) spacing between centers of dipoles: %8.6g [m]\n",XSpacing);
printf("Perpendicular (H-Plane) spacing between centers of dipoles: %8.6g [m]\n",YSpacing);
printf("Operating wavelength: %8.6g [m]\n",Lambda);
#endif

printf("\nZ12=%8.6g%+8.6gi [ohm] (referred to D1 input terminals)\n\n",Z12.r,Z12.i);

return 0;
}

/*****************************************************************************
* FUNC: void Quit(EVENT Evt, char *Message)
*
* DESC: Print Event message and exit to shell.
******************************************************************************/
void Quit(EVENT Evt, char *Message)
{

static char *Event[] = {

/*  NO_EVENT */       " ",
/*  MEMORY_ERR */     "Not enough memory",
/*  READ_OPEN_ERR */  "I can't find the file",
/*  READ_ERR */       "Error reading file",
/*  WRITE_OPEN_ERR */ "I can't open the file",
/*  WRITE_ERR */      "I can't write the file",
/*  BUFFER_ERR */     "I can't attach buffer to file",
/*  SIZE_ERR */       "Specified size doesn't match data size in file",
/*  MAXIT_ERR */      "Exceeded max number of iterations in function",
/*  LOGIC_ERR */      "Logic error in function",
/*  READ_NNET_ERR */  "Incompatible neural network in file",
/*  MTXDIM_ERR */     "Invalid matrix dimensions in file",
/*  PARF_ERR  */      "Wrong parameter file",
/*  GFTD_ERR  */      "I can't get time & date info from file",
/*  MAXFLOAT_READ_ERR */"Exceeded float maximum absolute value reading file",
/*---------*/

};

	system("cls");

	if (Evt != NO_EVENT) {


		if(Message[0]){
		fprintf (stderr,"%s %s!\n", Event[Evt], Message);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}
		else{
		fprintf (stderr,"%s!\n", Event[Evt]);
                #if(0)
		putch(BEL);
                #endif
		exit (Evt);
		}

	}
	exit (0);
}


/*****************************************************************************
* FUNC: inline void SiCi(double *Si, double *Ci,double x)
*
* DESC: The Sine Integral of x and the Cosine Inetgral of x are respectively
*       returned in *Si and *Ci.
******************************************************************************/
inline void SiCi(double *Si, double *Ci,double x)
{
	int i,k,odd;
	double a,err,fact,sign,sum,sumc,sums,t,term;
	complex h,b,c,d,del;
	t=fabs(x);
	if (t == 0.0) {
		*Si=0.0;
		*Ci = -1.0/FPMIN;
		return;
	}
	if (t > TMIN) {
		b=Complex(1.0,t);
		c=Complex(1.0/FPMIN,0.0);
		d=h=Cdiv(ONE,b);
		for (i=2;i<=SICIMAXIT;i++) {
			a = -(i-1)*(i-1);
			b=Cadd(b,Complex(2.0,0.0));
			d=Cdiv(ONE,Cadd(RCmul(a,d),b));
			c=Cadd(b,Cdiv(Complex(a,0.0),c));
			del=Cmul(c,d);
			h=Cmul(h,del);
			if (fabs(del.r-1.0)+fabs(del.i) < EPS) break;
		}
		if (i > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		h=Cmul(Complex(cos(t),-sin(t)),h);
		*Ci = -h.r;
		*Si=M_PI_2+h.i;
	} else {
		if (t < sqrt(FPMIN)) {
			sumc=0.0;
			sums=t;
		} else {
			sum=sums=sumc=0.0;
			sign=fact=1.0;
			odd=TRUE;
			for (k=1;k<=SICIMAXIT;k++) {
				fact *= t/k;
				term=fact/k;
				sum += sign*term;
				err=term/fabs(sum);
				if (odd) {
					sign = -sign;
					sums=sum;
					sum=sumc;
				} else {
					sumc=sum;
					sum=sums;
				}
				if (err < EPS) break;
				odd=!odd;
			}
			if (k > SICIMAXIT) Quit(MAXIT_ERR,"SiCi()");
		}
		*Si=sums;
		*Ci=sumc+log(t)+EULER;
	}
	if (x < 0.0) *Si = -(*Si);
}


/*****************************************************************************
* FUNC: inline complex Cadd(complex a, complex b)
*
* DESC: Return the sum of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cadd(complex a, complex b)
{
	complex c;
	c.r=a.r+b.r;
	c.i=a.i+b.i;
	return c;
}


/*****************************************************************************
* FUNC: inline complex Csub(complex a, complex b)
*
* DESC: Return the difference complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Csub(complex a, complex b)
{
	complex c;
	c.r=a.r-b.r;
	c.i=a.i-b.i;
	return c;
}



/*****************************************************************************
* FUNC: inline complex Cmul(complex a, complex b)
*
* DESC: Return the product of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cmul(complex a, complex b)
{
	complex c;
	c.r=a.r*b.r-a.i*b.i;
	c.i=a.i*b.r+a.r*b.i;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Complex(complex a, complex b)
*
* DESC: Return complex number re+j*im.
******************************************************************************/
inline complex Complex(double re, double im)
{
	complex c;
	c.r=re;
	c.i=im;
	return c;
}

/*****************************************************************************
* FUNC: inline complex Cdiv(complex a, complex b)
*
* DESC: Return the quotient of complex numbers 'a' and 'b'.
******************************************************************************/
inline complex Cdiv(complex a, complex b)
{
	complex c;
	double r,den;
	if (fabs(b.r) >= fabs(b.i)) {
		r=b.i/b.r;
		den=b.r+r*b.i;
		c.r=(a.r+r*a.i)/den;
		c.i=(a.i-r*a.r)/den;
	} else {
		r=b.r/b.i;
		den=b.i+r*b.r;
		c.r=(a.r*r+a.i)/den;
		c.i=(a.i*r-a.r)/den;
	}
	return c;
}


/*****************************************************************************
* FUNC: inline complex RCmul(double x, complex a)
*
* DESC: Return the product of real 'x' by complex 'a'.
******************************************************************************/
inline complex RCmul(double x, complex a)
{
	complex c;
	c.r=x*a.r;
	c.i=x*a.i;
	return c;
}


/*****************************************************************************
* FUNC: inline double Pythag(double a, double b)
*
* DESC: Compute (a^2+b^2)^{1/2} without overflow
*****************************************************************************/
inline double Pythag(double a, double b)
{
	double absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}

/*****************************************************************************
* FUNC: complex ZM_EchelonDipoles(float SL1, float SL2, float GSX, float GSY)
*
* DESC: Mutual Impedance of echelon Dipoles D1 and D2 with Symmetrical Feeding
*
* INPUT PARAMETERS:   SL1  - Half-length of Dipole 1 in Wavelengths.
*                     SL2  - Half-length of Dipole 2 in Wavelengths.
*                     GSX  - Parallel Center-to-Center spacing in Wavelengths (E-Plane).
*                     GSY  - Perpendicular spacing or dipole radius in Wavelengths (H-Plane).
*
* NOTE 1: Based upon the Induced FEM Method with current distribution
*         assumed to be sinusoidal (Thin Dipoles).
*
*
*                  SL1        SL1
*             __________   __________    ..................
*         D1            | |                               |
*                                                         |
*                        |
*                        |                               GSY
*                        |
*                        |        SL2       SL2           |
*                        |   D2 _______   _______  .......|
*                        |             | |
*                        |              |
*                        |------GSX-----|
*
* NOTE 2: The method is precise only for Thin Dipoles. A Thin Dipole is
*         such that the radius of the wire used to implement the dipole
*         is less than approximately (1/200) of the operating wavelength.
*
******************************************************************************/
complex ZM_EchelonDipoles(float SL1, float SL2, float GSX, float GSY)
{

double C_[3];
double *C;

complex E_[4];
complex *E;

complex SUM,ZP;

double T_[4];
double *T;

double A,B,G2,G3,Q,U,V, SL12, SL22;

complex F1,F2;

double D1,D2;

int M,N;

unsigned I;

double C1,S1,RA;


/* assign array init address to pointers */
C=C_;
E=E_;
T=T_;

/* backward unit-offset pointers for FORTRAN code compatibility */
C--;
E--;
T--;

SL12=(double)SL1;

SL22=(double)SL2;

G2=SQR(GSY);

G3=(double)GSX;

D1=sin(_2PI*SL12);

D2=sin(_2PI*SL22);

C[1]=1.0;

C[3]=1.0;

C[2]=-2.0*cos(_2PI*SL22);

SUM=Complex(0.0,0.0); /* SUM=0+j0 */

for(M=-1;M<=1;M++){

        for(N=-1;N<=1;N+=2){

        V=N*(G3+M*SL22);

        U=V+SL12;

        A=sqrt(G2+SQR(G3+N*SL12+M*SL22));

        B=sqrt(G2+SQR(G3+M*SL22));

        T[1]=_2PI*(A+U);

        T[2]=_2PI*(A-U);

        T[3]=_2PI*(B+V);

        T[4]=_2PI*(B-V);

                for(I=1;I<=4;I++){

                Q=fabs(T[I]);

                if(Q>0.0001){

                RA=T[I];

                SiCi(&S1,&C1,RA);

                }
                else{

                S1=T[I];

                C1=EULER+log(Q)-0.25*SQR(Q);

                }

                E[I]=Complex(C1,-S1); /* E[I]=C1-j*S1 */

                } /* for I */

F1=Complex(cos(_2PI*U),sin(_2PI*U)); /* F1= cos(_2PI*U)+j*sin(_2PI*U) */

F2=Complex(cos(_2PI*U),-sin(_2PI*U)); /* F2= cos(_2PI*U)-j*sin(_2PI*U) */

SUM=Cadd(SUM,RCmul(C[M+2],Cadd(Cmul(F1,Csub(E[1],E[3])),Cmul(F2,Csub(E[2],E[4]))))); /* SUM=SUM+C[M+2]*(F1*(E[1]-E[3])+F2*(E[2]-E[4])) */

        }/* for N */

}/* for M */

ZP=RCmul(15.0/(D1*D2),SUM);

/* zeroes pointers offset */
C++;
E++;
T++;

return ZP;

}

